<?php

namespace QUI\Controls\ChildrenList;

use QUI;
use QUI\Projects\Site\Utils;

/**
 * Class ChildrenList
 *
 * @package quiqqer/sitetypes
 */
class ChildrenList extends QUI\Control
{
    /**
     * constructor
     *
     * @param array $attributes
     */
    public function __construct($attributes = [])
    {
        // default options
        $this->setAttributes([
            'class'              => 'qui-controls-childrenlist',
            'limit'              => 6,
            'showItemAttributes' => [
                'title'       => true,
                'description' => true,
                'date'        => true,
                'image'       => true,
                'author'      => true,
                'url'         => true
            ],
            'moreBtnText'        => false,  // Individual text for the "more" btn for a single item
            'Site'               => false,
            'itemtype'           => 'https://schema.org/ItemList',
            'child-itemtype'     => 'https://schema.org/ListItem',
            'child-itemprop'     => 'itemListElement',
            // layout / design
            'display'            => 'childrenlist',
            // Custom children template (path to html file); overwrites "display"
            'displayTemplate'    => false,
            // Custom children template css (path to css file); overwrites "display"
            'displayCss'         => false,
            'nodeName'           => 'section',
            // list of sites to display,
            'children'           => [],
            // Instances of class \QUI\Controls\ChildrenList\ChildrenListItem
        ]);

        parent::__construct($attributes);
    }

    /**
     * Return the inner body of the element
     * Can be overwritten
     *
     * @return String
     *
     * @throws QUI\Exception
     */
    public function getBody()
    {
        try {
            $Engine = QUI::getTemplateManager()->getEngine();
        } catch (QUI\Exception $Exception) {
            QUI\System\Log::writeException($Exception);

            return '';
        }

        $Site = $this->getSite();

        $Pagination = new QUI\Controls\Navigating\Pagination();
        $Pagination->loadFromRequest();
        $Pagination->setAttribute('Site', $Site);

        $limit    = $this->getAttribute('limit');
        $children = $this->getAttribute('children');

        if (empty($children) || !\is_array($children)) {
            $children = [];
        } else {
            $children = \array_filter($children, function ($Child) {
                return $Child instanceof ChildrenListItem;
            });
        }

        $childrenCount = \count($children);

        // sheets
        $sheets = \ceil($childrenCount / $limit);

        $Pagination->setAttribute('limit', $limit);
        $Pagination->setAttribute('sheets', $sheets);

        $Engine->assign([
            'this'        => $this,
            'Site'        => $this->getSite(),
            'Project'     => $this->getProject(),
            'sheets'      => $sheets,
            'children'    => $children,
            'Pagination'  => $Pagination,
            'MetaList'    => new QUI\Controls\Utils\MetaList(),
            'Events'      => $this->Events,
            'show'        => $this->getAttribute('showItemAttributes'),
            'moreBtnText' => $this->getAttribute('moreBtnText') ?:
                QUI::getLocale()->get('quiqqer/controls', 'ChildrenList.btn.more')
        ]);

        // load custom template (if set)
        if ($this->getAttribute('displayTemplate')
            && \file_exists($this->getAttribute('displayTemplate'))
        ) {
            if ($this->getAttribute('displayCss')
                && \file_exists($this->getAttribute('displayCss'))
            ) {
                $this->addCSSFile($this->getAttribute('displayCss'));
            }

            return $Engine->fetch($this->getAttribute('displayTemplate'));
        }

        switch ($this->getAttribute('display')) {
            default:
            case 'childrenList':
                $css      = \dirname(__FILE__).'/ChildrenList.css';
                $template = \dirname(__FILE__).'/ChildrenList.html';
                break;

            // @todo all special types
//            case 'longFooter':
//                $css      = \dirname(__FILE__).'/ChildrenList.LongFooter.css';
//                $template = \dirname(__FILE__).'/ChildrenList.LongFooter.html';
//                break;
//
//            case 'authorTop':
//                $css      = \dirname(__FILE__).'/ChildrenList.AuthorTop.css';
//                $template = \dirname(__FILE__).'/ChildrenList.AuthorTop.html';
//                break;
//
//            case '1er':
//                $css      = \dirname(__FILE__).'/ChildrenList.1er.css';
//                $template = \dirname(__FILE__).'/ChildrenList.1er.html';
//                break;
//
//            case '2er':
//                $css      = \dirname(__FILE__).'/ChildrenList.2er.css';
//                $template = \dirname(__FILE__).'/ChildrenList.2er.html';
//                break;
//
//            case '3er':
//                $css      = \dirname(__FILE__).'/ChildrenList.3er.css';
//                $template = \dirname(__FILE__).'/ChildrenList.3er.html';
//                break;
//
//            case '4er':
//                $css      = \dirname(__FILE__).'/ChildrenList.4er.css';
//                $template = \dirname(__FILE__).'/ChildrenList.4er.html';
//                break;
//
//            case 'simpleArticleList':
//                $css      = \dirname(__FILE__).'/ChildrenList.SimpleArticleList.css';
//                $template = \dirname(__FILE__).'/ChildrenList.SimpleArticleList.html';
//                break;
//
//            case 'advancedArticleList':
//                $css      = \dirname(__FILE__).'/ChildrenList.AdvancedArticleList.css';
//                $template = \dirname(__FILE__).'/ChildrenList.AdvancedArticleList.html';
//                break;
//
//            case 'imageTopBorder':
//                $css      = \dirname(__FILE__).'/ChildrenList.ImageTopBorder.css';
//                $template = \dirname(__FILE__).'/ChildrenList.ImageTopBorder.html';
//                break;
//
//            case 'imageTop':
//                $css      = \dirname(__FILE__).'/ChildrenList.ImageTop.css';
//                $template = \dirname(__FILE__).'/ChildrenList.ImageTop.html';
//                break;
//
//            case 'cardRows':
//                $css      = \dirname(__FILE__).'/ChildrenList.CardRows.css';
//                $template = \dirname(__FILE__).'/ChildrenList.CardRows.html';
//                break;
//
//            case 'CSSGridCards':
//                $css      = \dirname(__FILE__).'/ChildrenList.CSSGridCards.css';
//                $template = \dirname(__FILE__).'/ChildrenList.CSSGridCards.html';
//                break;
        }

        $this->addCSSFile($css);

        return $Engine->fetch($template);
    }

    /**
     * @return mixed|QUI\Projects\Site
     */
    protected function getSite()
    {
        if ($this->getAttribute('Site')) {
            return $this->getAttribute('Site');
        }

        $Site = QUI::getRewrite()->getSite();

        $this->setAttribute('Site', $Site);

        return $Site;
    }
}
